from traceback import format_exc
from gauge_rules.genrules import genUciRules
from genBaselines import genUciBaselines

k_factor=1

def set_k_factor(kfactor=1):
    global k_factor
    try:
        k_factor=float(kfactor)
    except:
        k_factor=1
        print(format_exc())
        

def gen_pump_gauge(machine_id,category="OEM"):
    uci_rule=genUciRules()
    uci_baseline=genUciBaselines()
    return {
      "category": category,
      "mode": "24x7",
      "group": "24x7",
      "active": True,
      "id": machine_id,
      "blowerType": "PD (Straight 2-Lobe)",
      "gaugeUserInputs": uci_rule[machine_id] if machine_id in uci_rule else [],
      "baselines": uci_baseline[machine_id] if machine_id in uci_baseline else [],
      "eventInstances": []
    }

def gen_power_gauge(machine_id,category="OEM"):
    uci_rule=genUciRules()
    uci_baseline=genUciBaselines()
    return {
      "category": category,
      "mode": "24x7",
      "group": "24x7",
      "active": True,
      "id": machine_id,
      "kFactor": k_factor,
      "gaugeUserInputs": uci_rule[machine_id] if machine_id in uci_rule else [],
      "baselines": uci_baseline[machine_id] if machine_id in uci_baseline else [],
      "eventInstances": []
    }

def gen_gauges(uciMachines):
    gauges=[]
    for key,m in uciMachines.items():
        if m['type'] == 'pump':
            gauge=gen_pump_gauge(m['id'],category=m.get('manufacturer_type','OEM'))
        elif m['type'] == 'power':
            gauge=gen_power_gauge(m['id'],category=m.get('manufacturer_type','OEM'))
        elif m['type'] == 'rail':
            gauge=gen_pump_gauge(m['id'],category=m.get('manufacturer_type','OEM'))
        elif m['type'] == 'plc':
            gauge=gen_pump_gauge(m['id'],category=m.get('manufacturer_type','OEM'))    
        else:
            continue
        gauges.append(gauge)
    return gauges

def gen_machine_dh_detail(machine_id,name,category="OEM",group="24x7"):
    return {
        "name": name,
        "lowercase_name": name.lower(),
        "display_name": name,
        "description": "test",
        "category": category,
        "mode": group,
        "group": group,
        "id": machine_id,
    }        


def gen_dh_details(mac,uciMachines):
    machines=[]
    for key,m in uciMachines.items():
        machine=gen_machine_dh_detail(m['id'],m.get('name','Unknown'),category=m.get("manufacturer_type","OEM"))
        machines.append(machine)
    return {
        "name": "CloudTesting",
        "lowercase_name": "cloudtesting",
        "uuid": mac.replace(":",""),
        "macAddress": mac.lower(),
        "display_name": "CloudTesting",
        "dhVersion": 1121.00036,
        "createdOn": "2022-06-15T13:15:21.538Z",
        "machines": machines
    } 


def gen_vac_qc_param(sensor):
    id = sensor['id']
    #if "calib_a" not in sensor and "calib_b" not in sensor:
    #    return None
    a = sensor.get("calib_a")
    b = sensor.get("calib_b")
    calibration_parameters = {}
    if a:
        calibration_parameters['a'] = a
    if a:
        calibration_parameters['b'] = b
    return {
      "name": id,
      "lowercase_name": id.lower(),
      "display_name": id,
      "id": id,
      "source": "ble",
      "calibrationParam": {
        "calibration_parameters": calibration_parameters
      },
    }

def gen_plc_qc_param(sensor):
    id=sensor['id']
    id=id.split('_')[0]+"_PLC_"+str(int(sensor["port"])+1) if id.find("CDA")!=-1 else id
    return {
      "name": id,
      "lowercase_name": id.lower(),
      "display_name": id,
      "id": id,
      "source": "ble",
    }

def gen_pf_qc_param(sensor):
    id = sensor['id']
    param = {
      "name": id,
      "lowercase_name": id.lower(),
      "display_name": id.upper(),
      "id": id,
      "source": "gpio",
      "cttype": sensor.get('cttype',"100A"),
      "isostat": sensor.get('isostat',"No"),
      "loadtype": sensor.get('loadtype',"Normal"),
      "phasetype": sensor.get('phasetype',"Three"),
      "calibrationParam": { },
    }
    calibrationParams = {}
    calibration_parameters={}
    print("sensor: ", sensor)
    if 'voltage_phase1_a' in sensor:
        print("*********** Have Voltage phase 1 *************")
        calibration_parameters['voltage_phase1_a'] = {} 
        calibration_parameters['voltage_phase1_a']['v1'] = float(sensor['voltage_phase1_a'])
    if 'current_phase1_a' in sensor:
        calibration_parameters['current_phase1_a'] = {} 
        calibration_parameters['current_phase1_a']['i1'] = float(sensor['current_phase1_a'])
    if 'voltage_phase2_a' in sensor:
        calibration_parameters['voltage_phase2_a'] = {} 
        calibration_parameters['voltage_phase2_a']['v2'] = float(sensor['voltage_phase2_a'])
    if 'current_phase2_a' in sensor:
        calibration_parameters['current_phase2_a'] = {} 
        calibration_parameters['current_phase2_a']['i2'] = float(sensor['current_phase2_a'])
    if 'voltage_phase3_a' in sensor:
        calibration_parameters['voltage_phase3_a'] = {} 
        calibration_parameters['voltage_phase3_a']['v3'] = float(sensor['voltage_phase3_a'])
    if 'current_phase3_a' in sensor:
        calibration_parameters['current_phase3_a'] = {} 
        calibration_parameters['current_phase3_a']['i3'] = float(sensor['current_phase3_a'])
    if 'voltage_phase1_b' in sensor:
        calibration_parameters['voltage_phase1_b'] = {} 
        calibration_parameters['voltage_phase1_b']['v1'] = float(sensor['voltage_phase1_b'])
    if 'current_phase1_b' in sensor:
        calibration_parameters['current_phase1_b'] = {} 
        calibration_parameters['current_phase1_b']['i1'] = float(sensor['current_phase1_b'])

    if 'voltage_phase2_b' in sensor:
        calibration_parameters['voltage_phase2_b'] = {} 
        calibration_parameters['voltage_phase2_b']['v2'] = float(sensor['voltage_phase2_b'])
    if 'current_phase2_b' in sensor:
        calibration_parameters['current_phase2_b'] = {} 
        calibration_parameters['current_phase2_b']['i2'] = float(sensor['current_phase2_b'])
    if 'voltage_phase3_b' in sensor:
        calibration_parameters['voltage_phase3_b'] = {} 
        calibration_parameters['voltage_phase3_b']['v3'] = float(sensor['voltage_phase3_b'])
    if 'current_phase3_b' in sensor:
        calibration_parameters['current_phase3_b'] = {} 
        calibration_parameters['current_phase3_b']['i3'] = float(sensor['current_phase3_b'])

    print("calibration_parameters: ",calibration_parameters)
    if calibration_parameters:
        calibrationParams['calibration_parameters'] = calibration_parameters 
    param['calibrationParam'] = calibrationParams
    return param

def gen_qc_params(machines):
    qc_params=[]
    for cfg,m in machines.items():
        if 'sensors' not in m:
            continue
        for sensor_cfg,s in m['sensors'].items():
            qc = None
            if s['type'] == 'VAC':
                qc=gen_vac_qc_param(s)
            if s['type'] == 'PS':
                qc=gen_vac_qc_param(s)
            if s['type'] == 'PF':
                qc=gen_pf_qc_param(s)
            if s['type'] == 'PLC':
                qc=gen_plc_qc_param(s)
            if s['type'] == 'CDA':
                qc=gen_plc_qc_param(s)
            if qc:
                qc_params.append(qc)
    return qc_params
