import getmac
import sys
import simplejson as json
from machines import VACSensorPackage, STSensorPackage,PumpMachine
import getopt
from pynovatec.onboarding import onboard
from pynovatec.onboarding import cleanup_onboarding
from pynovatec import appconfig
from machineUci import genUciOnboard


#config = {
#   'interface': 'br-lan',
#   'base_path': '/opt/novatec/data/slave/json_data',
#   'sensor_config_file': '/opt/novatec/data/slave/conf/sensor_config.txt',
#   'calib_params_file': '/opt/novatec/data/slave/conf/calib_params.txt',
#   'frequency':600,
#   'api_host': 'api.machinesense.com',
#   'product_type':"ps"
#}

#config = {
#   'base_path': '/opt/novatec/data/slave/json_data',
#   'sensor_config_file': '/opt/novatec/data/slave/conf/sensor_config.txt',
#   'calib_params_file': '/opt/novatec/data/slave/conf/calib_params.txt',
#   'product_type':"ps"
#}
config_file=""
default_config = {
   'interface': 'br-lan',
   'base_path': '/opt/novatec/data/slave/json_data',
   'sensor_config_file': '/opt/novatec/data/slave/conf/sensor.config',
   'calib_params_file': '/opt/novatec/data/slave/conf/ps_calibration_details.conf',
   'product_type':"ps",
   'conf_dir': "/opt/novatec/data/slave/confs"
}



st="a1:1c:30:b9:d2:88"
vac="98:24:96:8e:08:a9"
mac=getmac.get_mac_address()
machine_id=None
machine_name = None


def gen_gauge(machine_id):
    return {
      "category": "OEM",
      "mode": "24x7",
      "group": "24x7",
      "active": True,
      "id": machine_id,
      "blowerType": "PD (Straight 2-Lobe)",
      "gaugeUserInputs": [],
      "eventInstances": []
    }


def gen_dh_details(mac,machine_id):
    return {
        "name": "CloudTesting",
        "lowercase_name": "cloudtesting",
        "uuid": "dca632ec71fe",
        "macAddress": mac.lower(),
        "display_name": "CloudTesting",
        "dhVersion": 1121.00036,
        "assignedDhVersion": 0,
        "id": "62a9db69c723e25c5e3a1a10",
        "locationId": "606495a65105f8d980f9efdf",
        "companyId": "58ff61f930075c2a48b5ca0b",
        "createdOn": "2022-06-15T13:15:21.538Z",
        "machines": [
          {
            "name": "CloudPump1",
            "lowercase_name": "cloudpump1",
            "display_name": "CloudPump1",
            "description": "test",
            "category": "OEM",
            "mode": "24x7",
            "group": "24x7",
            "qcprofile": {
              "start": 1655299063,
              "stop": 1655299064,
              "status": 21,
              "flagStateCount": 1,
              "events": [
                {
                  "t": 1655299063,
                  "c": 1
                },
                {
                  "t": 1655299064,
                  "c": 21
                }
              ]
            },
            "active": True,
            "serialnumber": "novatec-2022-03364",
            "id": machine_id,
            "zoneId": "606495c49d17742bb5be069b",
            "modelsId": "59aea70657df0a5217fb1d7f",
            "datahubId": "62a9db69c723e25c5e3a1a10",
            "blowerType": "PD (Straight 2-Lobe)",
            "vendorName": "Gardner Denver",
            "packageSerialNumber": "vpa-22-001770",
            "createdOn": "2022-06-15T13:17:42.332Z",
            "modifiedOn": "2022-06-15T13:48:43.884Z",
            "qcSerialNumber": "vpa-22-001770",
            "view_type": 1
          }
        ]
    } 


def usage():
    print(f"usage: {sys.argv[0]} [-?] [-m machineId] vac_mac st_mac")
    sys.exit()


try:
    opts, args = getopt.getopt(sys.argv[1:],"?M:-m:n:c:")
except getopt.GetoptError:
    usage()

MAC="30:24:a9:3d:54:ca"

for opt,arg in opts:
    #print("opt is: ",opt)
    if opt == "-M":
        machine_id = arg
    if opt == "-m":
        MAC = arg
    if opt == "-n":
        machine_name = arg
    if opt == "-c":
        config_file = arg
    if opt == "-?":
        usage()

appconfig.loadf(config_file,default_config)
config={
   'interface': appconfig.interface,
   'base_path': appconfig.base_path,
   'sensor_config_file': appconfig.sensor_config_file,
   'calib_params_file': appconfig.calib_params_file,
   'product_type':appconfig.product_type,
   'conf_dir': appconfig.conf_dir
}
onboard_json = genUciOnboard()
print(json.dumps(onboard_json,indent=4))
if onboard_json:
    try:
        onboard(onboard_json,**config)
    except Exception as e:
        pass
else:
  print("No machines found, cleaning up all onboarding, sensor and calibration details.")
  cleanup_onboarding(onboard_json,**config)
#print("args: ",args)



