from uci import UciExceptionNotFound
from EEUci import EEUci as Uci
import simplejson as json
from genDetails import gen_dh_details, gen_gauges, gen_qc_params, set_k_factor
import getmac
from machines import VACSensorPackage, STSensorPackage,PumpMachine,PowerMachine,PFSensorPackage,PSSensorPackage,RailCarMachine,PLCSensorPackage,PLCMachine
from machines import DryerSensorPackage, DryerMachine, CDASensorPackage, VFDISensorPackage, VFDOSensorPackage, ComponentAnalyzerSensorPackage, ComponentAnalyzerMachine
import sys

max_port=1

def get_max_port(port=1):
    global max_port
    try:
        max_port=max(max_port,int(port))
    except:
        print("Exception in max_port..")
        max_port=1

def check_dryer(subtype):
    return True if any([x in subtype for x in ["cassette","regen","process","cda"]]) else False

def check_component_analyzer(subtype):
    return True if subtype=="component_analyzer" else False

ca_index_count=0

def getSensorPackage(sensor,subtype,machinetype):
    global ca_index_count
    if sensor['type'].upper() == "VAC":
        return VACSensorPackage(sensor,subtype,machinetype)
    if sensor['type'].upper() == "PS":
        return PSSensorPackage(sensor,subtype,machinetype)
    if sensor['type'].upper() == "ST":
        if check_dryer(subtype):
            return DryerSensorPackage(sensor,subtype,machinetype)
        if check_component_analyzer(subtype):
            ca_index_count=ca_index_count+1
            return ComponentAnalyzerSensorPackage(sensor,subtype,machinetype,index=ca_index_count)
        return STSensorPackage(sensor,subtype,machinetype)
    if sensor.get('io_type','').lower() == "input":
        return VFDISensorPackage(sensor,subtype,machinetype)
    if sensor.get('io_type','').lower() == "output":
        return VFDOSensorPackage(sensor,subtype,machinetype)
    if sensor['type'].upper() == "PF":
        if check_dryer(subtype):
            return DryerSensorPackage(sensor,subtype,machinetype)
        return PFSensorPackage(sensor,subtype,machinetype)
    if sensor['type'].upper() == "PLC":
        return PLCSensorPackage(sensor,subtype,machinetype)
    if sensor['type'].upper() == "CDA":
        return CDASensorPackage(sensor,subtype,machinetype)
    
    
def genMachines(uciMachines):
    machines=[]
    for cfg,m in uciMachines.items():
        slist=[]
        subtype=m.get('subtype','motorSense')
        manufacturer_type=m.get('manufacturer_type','oem').lower()
        set_k_factor(m['k_factor'] if 'k_factor' in m else 1)
        for k,s in m['sensors'].items():
            p = getSensorPackage(s,subtype,m['type'])
            get_max_port(s.get('port',1))
            if p:
                slist.append(p)
        if m['type'] == 'pump':
            if check_dryer(subtype):
                r=DryerMachine(slist=slist,subtype=subtype).gen_machine(machine_id=m['id'],name=m.get('name',"Unknown"),category=manufacturer_type,pumpNumber=m.get('pump_number'),description=m.get('description','default'))
            elif check_component_analyzer(subtype):
                r=ComponentAnalyzerMachine(slist=slist,subtype=subtype).gen_machine(machine_id=m['id'],name=m.get('name',"Unknown"),category=manufacturer_type,pumpNumber=m.get('pump_number'),description=m.get('description','default'))
            else:
                r=PumpMachine(slist=slist).gen_machine(machine_id=m['id'],name=m.get('name',"Unknown"),category=manufacturer_type,pumpNumber=m.get('pump_number'),pump_model=m.get('pump_model'),blower_type=m.get('blower_type'),vendor_name=m.get('vendor_name'),blower_model=m.get('blower_model'),blower_serial_number=m.get('blower_serial_number'),part_number=m.get('part_number'),description=m.get('description','default'))
        elif m['type'] == 'rail':
            r=RailCarMachine(slist=slist).gen_machine(machine_id=m['id'],name=m.get('name',"Unknown"),category=manufacturer_type,pumpNumber=m.get('pump_number'),description=m.get('description','default'))
        elif m['type'] == 'power':
            if check_dryer(subtype):
                r=DryerMachine(slist=slist,subtype=subtype).gen_machine(machine_id=m['id'],name=m.get('name',"Unknown"),category=manufacturer_type,pumpNumber=m.get('pump_number'),description=m.get('description','default'))
            else:
                r=PowerMachine(slist=slist,subtype=subtype,max_port=max_port).gen_machine(machine_id=m['id'],name=m.get('name',"Unknown"),category=manufacturer_type,pumpNumber=m.get('pump_number'),description=m.get('description','default'))
        elif m['type'] == 'plc':
            r=PLCMachine(slist=slist).gen_machine(machine_id=m['id'],name=m.get('name',"Unknown"),category=manufacturer_type,pumpNumber=m.get('pump_number'),description=m.get('description','default'))
        else: 
            continue 
        machines.append(r)
    return machines

def get_sensors(u,machineId):
    sensor_list={}
    sensors = u.get("machines",stype="sensor")
    for skey,s in sensors.items():
        if s['machine'] == machineId:
            sensor_list[skey]=s
    return sensor_list


def getOnboardType(name):
    if name == 'power':
        return 'pa'
    elif name == 'pump':
        return 'ps'
    elif name == 'rail':
        return 'rc'
    elif name == 'plc':
        return 'plc'

    return 'ps'

def genUciOnboard():
    u = Uci()
    type = 'ps'
    print("Getting machines..")
    try:
        machines = u.get("machines",stype ="machine")
        print("Machines: ",json.dumps(machines,indent=4))
    except UciExceptionNotFound as e:
        return None

    if not machines:
        return None
    for cfg,m in machines.items():
       m["sensors"] = get_sensors(u,m['id'])

    print("Machines: ",json.dumps(machines,indent=4))

    d = next(iter(machines))
    print("d is: ",d)
    if not d:
        return None
        
    type = getOnboardType(machines[d]['type']) 
    gMachines=genMachines(machines)

    mac=getmac.get_mac_address(interface="eth0")
    print("mac: ",mac)
    dh_details = gen_dh_details(mac,machines)
    print("dh: ", json.dumps(dh_details,indent=4))
    gauges = gen_gauges(machines)
    print("gauges: ",json.dumps(gauges,indent=4))
    qc_params = gen_qc_params(machines)
    onboard_json = {                                     
        'type' : type,
        'machines': gMachines,
        'dh_details': dh_details,
        'gauges': gauges,
        'qc_params': qc_params
  
    }
    return onboard_json



